# NOTE: Derived from blib/lib/Convert/TNEF.pm.
# Changes made here will be lost when autosplit again.
# See AutoSplit.pm.
package Convert::TNEF;

#line 478 "blib/lib/Convert/TNEF.pm (autosplit into blib/lib/auto/Convert/TNEF/read.al)"
sub read {
 croak "Usage: Convert::TNEF->read(fh, parameters) "
  unless @_ == 2 or @_ == 3;
 my $self = shift;
 my $class = ref($self) || $self;
 $self = {};
 bless $self, $class;
 my ($fd, $parms) = @_;
 $fd = wraphandle($fd);

 my %parms = %dflts;
 @parms{keys %$parms} = values %$parms if defined $parms;
 $parms = \%parms;
 my $debug = $parms{debug};
 my $ignore_checksum = $parms{ignore_checksum};
 
 # Start of TNEF stream
 my $data;
 my $num_bytes = $fd->read($data, 4);
 return read_err($num_bytes,$fd,$!) unless equal($num_bytes,4);
 print "TNEF start: ",unpack("H*", $data),"\n" if $debug;
 return rtn_err("Not TNEF-encapsulated", $fd, $parms)
  unless $data eq $TNEF_SIGNATURE;

 # Key
 $num_bytes = $fd->read($data, 2);
 return read_err($num_bytes,$fd,$!) unless equal($num_bytes,2);
 print "TNEF key: ",unpack("H*", $data),"\n" if $debug;

 # Start of First Object
 $num_bytes = $fd->read($data, 1);
 return read_err($num_bytes,$fd,$!) unless equal($num_bytes,1);

 my $msg_att="";
 my $got_version = 0;
 my $att_cnt=0;

 my $is_msg = ($data eq $LVL_MESSAGE);
 my $is_att = ($data eq $LVL_ATTACHMENT);
 print "TNEF object start: ",unpack("H*", $data),"\n" if $debug;
 return rtn_err("Neither a message or an attachment", $fd, $parms)
  unless $is_msg or $is_att;

 my %msg=();
 my %msg_size=();
 my $is_eof = 0;
 while ($is_msg) {
  print "Message attribute\n" if $is_msg and $debug;
  $num_bytes = $fd->read($data, 4);
  return read_err($num_bytes,$fd,$!) unless equal($num_bytes,4);
  my $att_id = $data;

  print "TNEF message attribute: ",unpack("H*", $data),"\n" if $debug;
  return rtn_err("Bad Attribute found in Message", $fd, $parms)
   unless $att_name{$att_id};
  if ($att_id eq $att{TnefVersion}) {
   return rtn_err("Version must be first attribute in message", $fd, $parms)
    if $att_cnt;
   return rtn_err("Two attTnefVersion records found", $fd, $parms)
    if $got_version;
   $got_version = 1;
  }
  $att_cnt++;
  print "Got attribute:$att_name{$att_id}\n" if $debug;

  $num_bytes = $fd->read($data, 4);
  return read_err($num_bytes,$fd,$!) unless equal($num_bytes,4);

  print "HLength:", unpack("H8",$data), "\n" if $debug;
  my $length = unpack("V", $data);
  print "Length: $length\n" if $debug;

  # Get the attribute data (returns an object since data may
  # actually end up in a file)
  my $calc_chksum;
  $data=$self->build_data($fd, $length, \$calc_chksum, $parms) or return undef;
  $self->debug_print($length, $att_id, $data, $parms) if $debug;
  $msg{$att_name{$att_id}} = $data;

  $num_bytes = $fd->read($data, 2);
  return read_err($num_bytes,$fd,$!) unless equal($num_bytes,2);
  my $file_chksum = $data;
  if ($debug) {
   print "Calc Chksum:", unpack("H*",$calc_chksum),"\n";
   print "File Chksum:", unpack("H*",$file_chksum),"\n";
  }
  return rtn_err("Bad Checksum", $fd, $parms)
   unless $calc_chksum eq $file_chksum or $ignore_checksum;

  my $num_bytes=$fd->read($data, 1);
  # EOF (0 bytes) is ok
  return read_err($num_bytes,$fd,$!) unless defined $num_bytes;
  print "Next token:", unpack("H2", $data), "\n" if $debug;
  $is_msg = ($data eq $LVL_MESSAGE);
  $is_att = ($data eq $LVL_ATTACHMENT);
  $is_eof = ($num_bytes == 0);
 }

 return rtn_err("Not an attachment", $fd, $parms) unless $is_eof or $is_att;

 my @atts=();
 my @atts_size=();
 my %attachment=();
 my %attachment_size=();
 while ($is_att) {
  print "Attachment attribute\n" if $debug;

  $num_bytes = $fd->read($data, 4);
  return read_err($num_bytes,$fd,$!) unless equal($num_bytes,4);
  my $att_id = $data;

  print "TNEF attachment attribute: ",unpack("H*", $data),"\n" if $debug;
  return rtn_err("Bad Attribute found in attachment", $fd, $parms)
   unless $att_name{$att_id};
  return rtn_err("Version record found in attachment", $fd, $parms)
   if $att_id eq $att{TnefVersion};
  print "Got attribute:$att_name{$att_id}\n" if $debug;

  $num_bytes = $fd->read($data, 4);
  return read_err($num_bytes,$fd,$!) unless equal($num_bytes,4);

  print "HLength:", unpack("H8",$data), "\n" if $debug;
  my $length = unpack("V", $data);
  print "Length: $length\n" if $debug;

  # Get the attribute data (returns an object since data may
  # actually end up in a file)
  my $calc_chksum;
  $data=$self->build_data($fd, $length, \$calc_chksum, $parms) or return undef;
  $self->debug_print($length, $att_id, $data, $parms) if $debug;

  # See if we're starting a new attachment
  if ($att_id eq $att{AttachRenddata}) {
   # Save the previous attachment, if any
   if (%attachment) {
    $attachment{TN_Size} = {%attachment_size};
    my $attref = {%attachment};
    bless $attref, $class;
    push(@atts, $attref);
   }
   %attachment = (AttachRendData=>$data);
   %attachment_size = (AttachRendData=>$length);
  } else {
   $attachment{$att_name{$att_id}} = $data;
   $attachment_size{$att_name{$att_id}} = $length;
  }

  # Validate checksum
  $num_bytes = $fd->read($data, 2);
  return read_err($num_bytes,$fd,$!) unless equal($num_bytes,2);
  my $file_chksum = $data;
  print "Calc Chksum:", unpack("H*",$calc_chksum),"\n" if $debug;
  print "File Chksum:", unpack("H*",$file_chksum),"\n" if $debug;
  die "Bad Checksum\n" unless $calc_chksum eq $file_chksum or $ignore_checksum;

  my $num_bytes=$fd->read($data, 1);
  # EOF (0 bytes) is ok
  return read_err($num_bytes,$fd,$!) unless defined $num_bytes;
  print "Next token:", unpack("H2", $data), "\n" if $debug;
  $is_msg = ($data eq $LVL_MESSAGE);
  $is_att = ($data eq $LVL_ATTACHMENT);
  $is_eof = ($num_bytes < 1);
 }
 if (%attachment) {
  my $attref = {%attachment};
  my $attsiz = {%attachment_siz};
  bless $attref, $class;
  push(@atts, $attref);
  push(@atts_size, $attsiz);
 }

 return rtn_err("Message found in attachment section",$fd,$parms)
  if $is_msg;
 return rtn_err("Not an attachment", $fd, $parms) unless $is_eof or $is_att;
 print "EOF\n" if $debug and $is_eof;

 $msg{TN_Size} = \%msg_size;
 my $msg = \%msg;
 bless $msg, $class;
 $self->{TN_Message} = $msg;
 $self->{TN_Attachments} = \@atts;
 return $self;
}

# end of Convert::TNEF::read
1;
